<?php

namespace Ems\Controllers;

use Exception\Messages;
use Service\LogService;
use Service\OAuth\OAuthCustomImpl;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\ORM\EntityManager;


use Ems\Services\LancamentoLoteService;

class LancamentoLoteController {
    /** @var LancamentoLoteService $lancamentoLoteService */
    protected $lancamentoLoteService;

    /** @var EntityManager $entityManager */
    protected $entityManager;

    public function __construct(
        LancamentoLoteService $lancamentoLoteService,
        EntityManager $entityManager
    ) {
        $this->lancamentoLoteService = $lancamentoLoteService;
        $this->entityManager = $entityManager;
    }

    public function getLotesLancamento(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $cdOperador = (string) $row['CDOPERADOR'];
            $nrPeriodo = (int) $row['NRPERIODOAPURACAO'];

            $lotes = $this->lancamentoLoteService->
                getLotesLancamento($cdOperador, $nrPeriodo);

            $response->addDataSet(new DataSet('getLotesLancamento', $lotes));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getLancamentosEvento(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrLote = $row['NRLOTELANCEVENTO'];
            $nrEstrutura = $row['NRESTRUTURAM'];

            $lancamentos = $this->lancamentoLoteService->
                getLancamentosEvento($nrLote, $nrEstrutura);

            $response->addDataSet(new DataSet('getLancamentosEvento', $lancamentos));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getTiposMovimento(Request\Row $request, Response $response) {
        try {
            $tiposMov = $this->lancamentoLoteService->getTiposMovimento();

            $response->addDataSet(new DataSet('getTiposMovimento', $tiposMov));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function insertLoteLancamento(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $dsLote = (string) $row['DSLOTELANCEVENTO'];
            $dtCompetencia = (string) $row['DTCOMPETLOTEEVE'];
            $cdOperador = (string) $row['CDOPERADOR'];
            $nrTipoMov = (int) $row['NRTIPOMOVIMENT'];
            $dtOcorrencia = (string) $row['DTOCORRENCIA'];

            $success = $this->lancamentoLoteService->insertLoteLancamento(
                $dsLote, $dtOcorrencia, $dtCompetencia,
                $cdOperador, $nrTipoMov
            );

            $dataSet = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('insertLoteLancamento', $dataSet));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getEventosLancamento(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrEstrutura = $row['NRESTRUTURAM'];

            $eventos = $this->lancamentoLoteService->
                getEventosLancamento($nrEstrutura);

            $response->addDataSet(new DataSet('getEventosLancamento', $eventos));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function editLoteLancamento(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrLote = (int) $row['NRLOTELANCEVENTO'];
            $dsLote = (string) $row['DSLOTELANCEVENTO'];
            $dtCompetencia = (string) $row['DTCOMPETLOTEEVE'];
            $cdOperador = (string) $row['CDOPERADOR'];
            $nrTipoMov = (int) $row['NRTIPOMOVIMENT'];
            $dtOcorrencia = (string) $row['DTOCORRENCIA'];

            $success = $this->lancamentoLoteService->editLoteLancamento(
                $nrLote, $dsLote, $dtOcorrencia, $dtCompetencia,
                $cdOperador, $nrTipoMov
            );

            $dataSet = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('editLoteLancamento', $dataSet));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function confirmLoteLancamento(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrLote = (int) $row['NRLOTELANCEVENTO'];

            $success = $this->lancamentoLoteService->confirmLoteLancamento($nrLote);

            $dataSet = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('confirmLoteLancamento', $dataSet));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function editLancamentos(Request\Row $request, Response $response) {
        $startedTransaction = false;

        try {
            $row = $request->getRow();
            $lancamentos = (array) $row['lancamentos'];
            $cdOperador = (string) $row['CDOPERADOR'];

            $this->entityManager->beginTransaction();
            $startedTransaction = true;

            $success = $this->lancamentoLoteService->editLancamentos($lancamentos, $cdOperador);

            if ($success) {
                $this->entityManager->commit();
            } else {
                throw new \Exception('Não foi possível salvar os lançamentos.');
            }

            $dataSet = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('editLancamentos', $dataSet));
        } catch(\Exception $e) {
            if ($startedTransaction) {
                $this->entityManager->rollback();
            }

            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function deleteLoteLancamento(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrLote = (int) $row['NRLOTELANCEVENTO'];

            $this->entityManager->beginTransaction();

            $success = $this->lancamentoLoteService->deleteLoteLancamento($nrLote);

            if ($success) {
                $this->entityManager->commit();
            } else {
                throw new \Exception('Não foi possível salvar os lançamentos.');
            }

            $dataSet = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('deleteLoteLancamento', $dataSet));
        } catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
}
