<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

use Util\Environment;
use Util\Repositories;
use Ems\Util\Utils;

use Ems\Repository\ParamOrgRepository;

class ParamOrgService {
    /** @param EntityManager */
    protected $entityManager;

    /** @param Environment */
    protected $environment;

    /** @param ParamOrgRepository */
    protected $paramOrgRepository;

    public function __construct(
        EntityManager $entityManager,
        Environment $environment,
        ParamOrgRepository $paramOrgRepository
    ) {
        $this->entityManager        = $entityManager;
        $this->environment          = $environment;
        $this->paramOrgRepository   = $paramOrgRepository;
    }

    private const NRPERFACESSADM_INDEX = 'NRPERFACESSADM';

    public const PERFIL_ACESS_ADM_DP_RH = 'PERFIL_ACESS_ADM_DP_RH';
    
    public const IDAPROVAOCORRENCIA_TRUE = 'S';
    public const IDAPROVAOCORRENCIA_FALSE = 'N';
    public const IDAPROVAOCORRENCIA_DEFAULT_VALUE = self::IDAPROVAOCORRENCIA_FALSE;

    public function getPerfilAcessoAdministradorOrg() {
        $nrorg = $this->environment->getNrorg();

        $paramOrg = $this->paramOrgRepository->getParamOrg($nrorg);

        if (!empty($paramOrg)) {
            return $paramOrg[self::NRPERFACESSADM_INDEX];
        }

        return false;
    }

    public function getPerfilAcessoOrg(string $perfilAcesso = self::PERFIL_ACESS_ADM_DP_RH) {
        $nrorg = $this->environment->getNrorg();

        $paramOrg = $this->paramOrgRepository->getParamOrg($nrorg);

        if (!empty($paramOrg)) {
            switch ($perfilAcesso) {
                case self::PERFIL_ACESS_ADM_DP_RH:
                    return $paramOrg[self::NRPERFACESSADM_INDEX];
                    break;

            }
        }

        return false;
    }

    public function getParamOrg() {
        $nrorg = $this->environment->getNrorg();

        return $this->entityManager->getRepository(Repositories::GPE_PARAMORG)->findOneBy(array('nrorg'=>$nrorg));
    }

    public function paramOrgValidaHoraBasePortal(
        ?int $nrVinculo = null
    ): bool {
        $paramOrg = $this->getParamOrg();
        if (empty($paramOrg)) {
            return false;
        }

        if (!empty($nrVinculo) && $this->isValidacaoHorarioBaseNecessariaExcecao($nrVinculo)) {
            return true;
        }

        return $paramOrg->getIdvalidahorabaseportal() == Utils::SIM;
    }

    /**
     * @return array|false
     */
    public function getValorReferencia(
        string $dsReferencia,
        string $dtCompetencia = null
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();
        $dtCompetencia = $dtCompetencia ?? date('01/m/Y');

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_DTMESCOMPETENC'  => $dtCompetencia,
            'P_DSREFERENCIA'    => $dsReferencia,
        ];

        $valorReferencia = $this->paramOrgRepository->getValorReferencia($params);

        return $valorReferencia;
    }

    public function get() {
        $nrOrg = (int) $this->environment->getNrOrg();
        $paramOrg = $this->paramOrgRepository->getParamOrg($nrOrg);

        return $paramOrg;
    }
    
    /**
     *  Função responsável por verificar se, nos parametros da organizacao (GPE_PARAMORG) se gestores comuns estão autorizados a aprovar ocorrencias
     * 
     */
    public function isIdAprovOcorrencia() {
        $paramOrg = $this->get();
        $idAprovaOcorrencia = isset($paramOrg) ? $paramOrg['IDAPROVOCORRENCIA'] == self::IDAPROVAOCORRENCIA_TRUE : self::IDAPROVAOCORRENCIA_DEFAULT_VALUE;
        return $idAprovaOcorrencia;
    }
    
    public function getColetorPF(): ?int {
        $paramOrg = $this->get();
        return $paramOrg['NRCOLETORFUNCIO'] ?? null;
    }
    
    public function getColetorPG(): ?int {
        $paramOrg = $this->get();
        return $paramOrg['NRCOLETORGESTOR'] ?? null;
    }

    public function getColetorAutomatica(): ?int {
        $paramOrg = $this->get();
        return $paramOrg['NRCOLETORMARCAUTOM'] ?? null;
    }
    
    public function getColetorGeradaPeloSistema(): ?int {
        $paramOrg = $this->get();
        return $paramOrg['NRCOLETORMARCGERSIST'] ?? null;
    }
    
    public function getColetorViradaDia(): ?int {
        $paramOrg = $this->get();
        return $paramOrg['NRCOLETORMARCVIRADIA'] ?? null;
    }
    
    public function getColetorProjetada(): ?int {
        $paramOrg = $this->get();
        return $paramOrg['NRCOLETORMARCPROJETADA'] ?? null;
    }
    
    public function isAprovacaoMarcNecessaria(): ?bool {
        $paramOrg = $this->get();
        return ($paramOrg['IDEXIGEAPROVMARCPF'] ?? null) == Utils::SIM;
    }
    
    public function isConfirmacaoMarcNecessaria(): ?bool {
        $paramOrg = $this->get();
        return ($paramOrg['IDEXIGECONFMARCPF'] ?? null) == Utils::SIM;
    }

    public function isSetBloqueioPGGestorEdicaoMarc(): ?bool {
        // Futuro parametro da organizacao
        switch ($this->environment->getNrOrg()) {
            case 3260:
                return true;
            default:
                return false;
        }
    }

    public function isValidacaoHorarioBaseNecessariaExcecao(
        int $nrVinculo
    ): bool {
        return $this->environment->getNrOrg() == Utils::ORG_TEKNISA &&
            Utils::getInstanceByClassName(VinculoService::class)->isVinculoEstagiario($nrVinculo);
    }

    public static function getInstance() {
        $instanceManager = InstanceManager::getInstance();
        return $instanceManager->getService('ParamOrgService');
    }
}